@@ -83,7 +83,7 @@ def fetch(ui, repo, source='default', **opts):
         # Is this a simple fast-forward along the current branch?
         newheads = repo.branchheads(branch)
         newchildren = repo.changelog.nodesbetween([parent], newheads)[2]
-        if len(newheads) == 1:
+        if len(newheads) == 1 and len(newchildren):
             if newchildren[0] != parent:
                 return hg.clean(repo, newchildren[0])
             else:
@@ -102,6 +102,9 @@ def fetch(ui, repo, source='default', **opts):
                       (len(newheads) - 1))
             return 1
 
+        if not newheads:
+            return 0
+
         # Otherwise, let's merge.
         err = False
         if newheads:
