@@ -204,6 +204,13 @@ class AbstractMethodException(Exception):
 class BadTypeInNameException(Exception):
 	pass
 
+class BadDomainName(Exception):
+	def __init__(self, pos):
+		Exception.__init__(self, "at position %s" % pos)
+
+class BadDomainNameCircular(BadDomainName):
+	pass
+
 # implementation classes
 
 class DNSEntry(object):
@@ -230,14 +237,14 @@ class DNSEntry(object):
 		"""Class accessor"""
 		try:
 			return _CLASSES[clazz]
-		except:
+		except KeyError:
 			return "?(%s)" % (clazz)
 
 	def getType(self, type):
 		"""Type accessor"""
 		try:
 			return _TYPES[type]
-		except:
+		except KeyError:
 			return "?(%s)" % (type)
 
 	def toString(self, hdr, other):
@@ -353,7 +360,7 @@ class DNSAddress(DNSRecord):
 		"""String representation"""
 		try:
 			return socket.inet_ntoa(self.address)
-		except:
+		except Exception:
 			return self.address
 
 class DNSHinfo(DNSRecord):
@@ -560,7 +567,7 @@ class DNSIncoming(object):
 				#
 				#print "UNKNOWN TYPE = " + str(info[0])
 				#raise BadTypeInNameException
-				pass
+				self.offset += info[3]
 
 			if rec is not None:
 				self.answers.append(rec)
@@ -575,8 +582,7 @@ class DNSIncoming(object):
 
 	def readUTF(self, offset, len):
 		"""Reads a UTF-8 string of a given length from the packet"""
-		result = self.data[offset:offset+len].decode('utf-8')
-		return result
+		return self.data[offset:offset+len].decode('utf-8')
 
 	def readName(self):
 		"""Reads a domain name from the packet"""
@@ -585,7 +591,7 @@ class DNSIncoming(object):
 		next = -1
 		first = off
 
-		while 1:
+		while True:
 			len = ord(self.data[off])
 			off += 1
 			if len == 0:
@@ -599,10 +605,10 @@ class DNSIncoming(object):
 					next = off + 1
 				off = ((len & 0x3F) << 8) | ord(self.data[off])
 				if off >= first:
-					raise "Bad domain name (circular) at " + str(off)
+					raise BadDomainNameCircular(off)
 				first = off
 			else:
-				raise "Bad domain name at " + str(off)
+				raise BadDomainName(off)
 
 		if next >= 0:
 			self.offset = next
@@ -784,7 +790,7 @@ class DNSCache(object):
 		"""Adds an entry"""
 		try:
 			list = self.cache[entry.key]
-		except:
+		except KeyError:
 			list = self.cache[entry.key] = []
 		list.append(entry)
 
@@ -793,7 +799,7 @@ class DNSCache(object):
 		try:
 			list = self.cache[entry.key]
 			list.remove(entry)
-		except:
+		except KeyError:
 			pass
 
 	def get(self, entry):
@@ -802,7 +808,7 @@ class DNSCache(object):
 		try:
 			list = self.cache[entry.key]
 			return list[list.index(entry)]
-		except:
+		except (KeyError, ValueError):
 			return None
 
 	def getByDetails(self, name, type, clazz):
@@ -815,7 +821,7 @@ class DNSCache(object):
 		"""Returns a list of entries whose key matches the name."""
 		try:
 			return self.cache[name]
-		except:
+		except KeyError:
 			return []
 
 	def entries(self):
@@ -823,7 +829,7 @@ class DNSCache(object):
 		def add(x, y): return x+y
 		try:
 			return reduce(add, self.cache.values())
-		except:
+		except Exception:
 			return []
 
 
@@ -863,9 +869,10 @@ class Engine(threading.Thread):
 					for socket in rr:
 						try:
 							self.readers[socket].handle_read()
-						except:
-							traceback.print_exc()
-				except:
+						except Exception:
+							if not globals()['_GLOBAL_DONE']:
+								traceback.print_exc()
+				except Exception:
 					pass
 
 	def getReaders(self):
@@ -932,7 +939,7 @@ class Reaper(threading.Thread):
 		self.start()
 
 	def run(self):
-		while 1:
+		while True:
 			self.zeroconf.wait(10 * 1000)
 			if globals()['_GLOBAL_DONE']:
 				return
@@ -981,7 +988,7 @@ class ServiceBrowser(threading.Thread):
 					callback = lambda x: self.listener.removeService(x, self.type, record.alias)
 					self.list.append(callback)
 					return
-			except:
+			except Exception:
 				if not expired:
 					self.services[record.alias.lower()] = record
 					callback = lambda x: self.listener.addService(x, self.type, record.alias)
@@ -996,7 +1003,7 @@ class ServiceBrowser(threading.Thread):
 		self.zeroconf.notifyAll()
 
 	def run(self):
-		while 1:
+		while True:
 			event = None
 			now = currentTimeMillis()
 			if len(self.list) == 0 and self.nextTime > now:
@@ -1060,16 +1067,16 @@ class ServiceInfo(object):
 			for key in properties:
 				value = properties[key]
 				if value is None:
-					suffix = ''.encode('utf-8')
+					suffix = ''
 				elif isinstance(value, str):
-					suffix = value.encode('utf-8')
+					suffix = value
 				elif isinstance(value, int):
 					if value:
 						suffix = 'true'
 					else:
 						suffix = 'false'
 				else:
-					suffix = ''.encode('utf-8')
+					suffix = ''
 				list.append('='.join((key, suffix)))
 			for item in list:
 				result = ''.join((result, struct.pack('!c', chr(len(item))), item))
@@ -1110,7 +1117,7 @@ class ServiceInfo(object):
 					result[key] = value
 
 			self.properties = result
-		except:
+		except Exception:
 			traceback.print_exc()
 			self.properties = None
 
@@ -1248,7 +1255,7 @@ class Zeroconf(object):
 		try:
 			self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
 			self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
-		except:
+		except Exception:
 			# SO_REUSEADDR should be equivalent to SO_REUSEPORT for
 			# multicast UDP sockets (p 731, "TCP/IP Illustrated,
 			# Volume 2"), but some BSD-derived systems require
@@ -1263,7 +1270,7 @@ class Zeroconf(object):
 		self.socket.setsockopt(socket.SOL_IP, socket.IP_MULTICAST_LOOP, 1)
 		try:
 			self.socket.bind(self.group)
-		except:
+		except Exception:
 			# Some versions of linux raise an exception even though
 			# the SO_REUSE* options have been set, so ignore it
 			#
@@ -1363,7 +1370,7 @@ class Zeroconf(object):
 				self.servicetypes[info.type]-=1
 			else:
 				del self.servicetypes[info.type]
-		except:
+		except KeyError:
 			pass
 		now = currentTimeMillis()
 		nextTime = now
@@ -1448,7 +1455,7 @@ class Zeroconf(object):
 		try:
 			self.listeners.remove(listener)
 			self.notifyAll()
-		except:
+		except Exception:
 			pass
 
 	def updateRecord(self, now, rec):
@@ -1521,7 +1528,7 @@ class Zeroconf(object):
 						out.addAnswer(msg, DNSText(question.name, _TYPE_TXT, _CLASS_IN | _CLASS_UNIQUE, _DNS_TTL, service.text))
 					if question.type == _TYPE_SRV:
 						out.addAdditionalAnswer(DNSAddress(service.server, _TYPE_A, _CLASS_IN | _CLASS_UNIQUE, _DNS_TTL, service.address))
-				except:
+				except Exception:
 					traceback.print_exc()
 
 		if out is not None and out.answers:
@@ -1534,7 +1541,7 @@ class Zeroconf(object):
 		#temp = DNSIncoming(out.packet())
 		try:
 			self.socket.sendto(out.packet(), 0, (addr, port))
-		except:
+		except Exception:
 			# Ignore this, it may be a temporary loss of network connection
 			pass
 
@@ -1571,3 +1578,5 @@ if __name__ == '__main__':
 	r.unregisterService(info)
 	print "   Unregister done."
 	r.close()
+
+# no-check-code
