@@ -64,11 +64,16 @@ class dirstate(object):
         self._read()
         return self._copymap
 
+    @propertycache
+    def _normroot(self):
+        return util.normcase(self._root)
+
     @propertycache
     def _foldmap(self):
         f = {}
         for name in self._map:
             f[util.normcase(name)] = name
+        f['.'] = '.' # prevents useless util.fspath() invocation
         return f
 
     @propertycache
@@ -383,7 +388,7 @@ class dirstate(object):
                 folded = path
             else:
                 folded = self._foldmap.setdefault(normed,
-                                util.fspath(path, self._root))
+                                util.fspath(normed, self._normroot))
         return folded
 
     def normalize(self, path, isknown=False):
