@@ -41,7 +41,7 @@ def _updatesample(dag, nodes, sample, always, quicksamplesize=0):
 def _setupsample(dag, nodes, size):
     if len(nodes) <= size:
         return set(nodes), None, 0
-    always = set(dag.heads())
+    always = dag.headsetofconnecteds(nodes)
     desiredlen = size - len(always)
     if desiredlen <= 0:
         # This could be bad if there are very many heads, all unknown to the
@@ -128,7 +128,7 @@ def findcommonheads(ui, local, remote,
         return (srvheadhashes, False, srvheadhashes,)
 
     if sample and util.all(yesno):
-        ui.note("all local heads known remotely\n")
+        ui.note(_("all local heads known remotely\n"))
         ownheadhashes = dag.externalizeall(ownheads)
         return (ownheadhashes, True, srvheadhashes,)
 
@@ -158,7 +158,7 @@ def findcommonheads(ui, local, remote,
             break
 
         if full:
-            ui.note("sampling from both directions\n")
+            ui.note(_("sampling from both directions\n"))
             sample = _takefullsample(dag, undecided, size=fullsamplesize)
         elif common:
             # use cheapish initial sample
