@@ -4,6 +4,9 @@ if 'TERM' in os.environ:
     del os.environ['TERM']
 import doctest
 
+import mercurial.util
+doctest.testmod(mercurial.util)
+
 import mercurial.changelog
 doctest.testmod(mercurial.changelog)
 
@@ -22,9 +25,6 @@ doctest.testmod(mercurial.ui)
 import mercurial.url
 doctest.testmod(mercurial.url)
 
-import mercurial.util
-doctest.testmod(mercurial.util)
-
 import mercurial.encoding
 doctest.testmod(mercurial.encoding)
 
@@ -33,3 +33,9 @@ doctest.testmod(mercurial.hgweb.hgwebdir_mod)
 
 import hgext.convert.cvsps
 doctest.testmod(hgext.convert.cvsps)
+
+import mercurial.revset
+doctest.testmod(mercurial.revset)
+
+import mercurial.minirst
+doctest.testmod(mercurial.minirst)
